@@ -42,10 +42,16 @@ typedef enum {
   PANGO_ALIGN_RIGHT
 } PangoAlignment;
 
+typedef enum {
+  PANGO_WRAP_WORD,
+  PANGO_WRAP_CHAR
+} PangoWrapMode;
+
 struct _PangoLayoutLine
 {
   PangoLayout *layout;
-  gint         length;		/* length of line in bytes*/
+  gint         start_index;     /* start of line as byte index into layout->text */
+  gint         length;		/* length of line in bytes */
   GSList      *runs;
 };
 
@@ -69,6 +75,7 @@ struct _PangoLayoutRun
 
 GType        pango_layout_get_type       (void) G_GNUC_CONST;
 PangoLayout *pango_layout_new            (PangoContext   *context);
+PangoLayout *pango_layout_copy           (PangoLayout    *src);
 
 PangoContext  *pango_layout_get_context    (PangoLayout    *layout);
 
@@ -96,6 +103,9 @@ void           pango_layout_set_font_description (PangoLayout                *la
 void           pango_layout_set_width            (PangoLayout                *layout,
 						  int                         width);
 int            pango_layout_get_width            (PangoLayout                *layout);
+void           pango_layout_set_wrap             (PangoLayout                *layout,
+                                                  PangoWrapMode               wrap);
+PangoWrapMode  pango_layout_get_wrap             (PangoLayout                *layout);
 void           pango_layout_set_indent           (PangoLayout                *layout,
 						  int                         indent);
 int            pango_layout_get_indent           (PangoLayout                *layout);
@@ -158,7 +168,7 @@ GSList *         pango_layout_get_lines            (PangoLayout    *layout);
 
 void     pango_layout_line_ref          (PangoLayoutLine  *line);
 void     pango_layout_line_unref        (PangoLayoutLine  *line);
-void     pango_layout_line_x_to_index   (PangoLayoutLine  *line,
+gboolean pango_layout_line_x_to_index   (PangoLayoutLine  *line,
 					 int               x_pos,
 					 int              *index,
 					 int              *trailing);
