@@ -29,6 +29,7 @@
 #include "pango-utils.h"
 #include "pangox-private.h"
 #include "pango-intset.h"
+#include "modules.h"
 
 #define PANGO_LIGATURE_HACK_DEBUG
 
@@ -199,7 +200,7 @@ pango_x_find_subfont (PangoFont  *font,
   
   if (subfont_index < 1 || subfont_index > xfont->n_subfonts)
     {
-      g_warning ("Error parsing ligature info: Invalid subfont %d", subfont_index);
+      g_warning ("Invalid subfont %d", subfont_index);
       return NULL;
     }
   
@@ -216,7 +217,7 @@ pango_x_make_font_struct (PangoFont *font, PangoXSubfontInfo *info)
   
   info->font_struct = pango_x_font_cache_load (cache, info->xlfd);
   if (!info->font_struct)
-    g_warning ("Error parsing ligature info: Cannot load font for XLFD '%s\n", info->xlfd);
+    g_warning ("Cannot load font for XLFD '%s\n", info->xlfd);
   
   info->is_1byte = (info->font_struct->min_byte1 == 0 && info->font_struct->max_byte1 == 0);
   info->range_byte1 = info->font_struct->max_byte1 - info->font_struct->min_byte1 + 1;
@@ -246,9 +247,19 @@ pango_x_get_context (Display *display)
 {
   PangoContext *result;
   PangoXContextInfo *info;
-
-  g_return_val_if_fail (display != NULL, NULL);
-
+  int i;
+  static gboolean registered_modules = FALSE;
+  
+  g_return_val_if_fail (display != NULL, NULL);  
+  
+  if (!registered_modules)
+    {
+      registered_modules = TRUE;
+      
+      for (i = 0; _pango_included_x_modules[i].list; i++)
+        pango_module_register (&_pango_included_x_modules[i]);
+    }
+  
   result = pango_context_new ();
   
   info = g_new (PangoXContextInfo, 1);
@@ -511,6 +522,30 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
     }
 }
 
+static gboolean
+get_int_prop (Atom         atom,
+              XFontStruct *fs,
+              int         *val)
+{
+  int i;
+
+  *val = 0;
+  
+  i = 0;
+  while (i < fs->n_properties)
+    {
+      if (fs->properties[i].name == atom)
+        {
+          *val = fs->properties[i].card32;
+          return TRUE;
+        }
+
+      ++i;
+    }
+
+  return FALSE;
+}
+
 /* Get composite font metrics for all subfonts in list
  */
 static void
@@ -518,8 +553,15 @@ get_font_metrics_from_subfonts (PangoFont        *font,
 				GSList           *subfonts,
 				PangoFontMetrics *metrics)
 {
+  PangoXFont *xfont = (PangoXFont *)font;
   GSList *tmp_list = subfonts;
   gboolean first = TRUE;
+  int total_avg_widths = 0;
+  int n_avg_widths = 0;
+  Atom avg_width_atom;
+
+  avg_width_atom = pango_x_fontmap_atom_from_name (xfont->fontmap,
+                                                   "AVERAGE_WIDTH");
   
   metrics->ascent = 0;
   metrics->descent = 0;
@@ -531,6 +573,8 @@ get_font_metrics_from_subfonts (PangoFont        *font,
       if (subfont)
 	{
 	  XFontStruct *fs = pango_x_get_font_struct (font, subfont);
+          gint avg_width;
+          
 	  if (fs)
 	    {
 	      if (first)
@@ -545,12 +589,21 @@ get_font_metrics_from_subfonts (PangoFont        *font,
 		  metrics->descent = MAX (fs->descent * PANGO_SCALE, metrics->descent);
 		}
 	    }
+
+          if (!get_int_prop (avg_width_atom, fs, &avg_width))
+            avg_width = (fs->min_bounds.width + fs->max_bounds.width) / 2;
+
+          total_avg_widths += avg_width;
+          n_avg_widths += 1;
 	}
       else
-	g_warning ("Error parsing ligature info: Invalid subfont %d in get_font_metrics_from_subfonts", GPOINTER_TO_UINT (tmp_list->data));
+	g_warning ("Invalid subfont %d in get_font_metrics_from_subfonts", GPOINTER_TO_UINT (tmp_list->data));
 	  
       tmp_list = tmp_list->next;
     }
+
+  /* This is pretty darn bogus. */
+  metrics->approximate_char_width = total_avg_widths / n_avg_widths;
 }
 
 /* Get composite font metrics for all subfonts resulting from shaping
@@ -576,9 +629,7 @@ get_font_metrics_from_string (PangoFont        *font,
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   GSList *subfonts = NULL;
   
-  n_chars = g_utf8_strlen (str, -1);
-
-  text_ucs4 = g_utf8_to_ucs4 (str, strlen (str));
+  text_ucs4 = g_utf8_to_ucs4_fast (str, -1, &n_chars);
   if (!text_ucs4)
     return;
 
@@ -729,12 +780,32 @@ pango_x_font_get_metrics (PangoFont        *font,
 
   if (!tmp_list)
     {
+      PangoLayout *layout;
+      PangoRectangle extents;
+      PangoContext *context;
+      
       info = g_new (PangoXMetricsInfo, 1);
       info->lang = lookup_lang;
 
       xfont->metrics_by_lang = g_slist_prepend (xfont->metrics_by_lang, info);
       
       get_font_metrics_from_string (font, lang, str, &info->metrics);
+
+      /* This is sort of a sledgehammer solution, but we cache this
+       * stuff so not a huge deal hopefully. Get the avg. width of the
+       * chars in "0123456789"
+       */
+      context = pango_x_get_context (pango_x_fontmap_get_display (xfont->fontmap));
+      pango_context_set_lang (context, lang);
+      layout = pango_layout_new (context);
+      pango_layout_set_text (layout, "0123456789", -1);
+
+      pango_layout_get_extents (layout, NULL, &extents);
+      
+      info->metrics.approximate_digit_width = extents.width / 10.0;
+
+      g_object_unref (G_OBJECT (layout));
+      g_object_unref (G_OBJECT (context));
     }
       
   *metrics = info->metrics;
@@ -989,7 +1060,7 @@ pango_x_font_subfont_xlfd (PangoFont     *font,
   subfont = pango_x_find_subfont (font, subfont_id);
   if (!subfont)
     {
-      g_warning ("Error parsing ligature info: pango_x_font_subfont_xlfd: Invalid subfont_id specified");
+      g_warning ("pango_x_font_subfont_xlfd: Invalid subfont_id specified");
       return NULL;
     }
 
@@ -1281,7 +1352,7 @@ pango_x_render_layout_line (Display          *display,
       pango_x_get_item_properties (run->item, &uline, &fg_color, &fg_set, &bg_color, &bg_set);
 
       if (fg_set && info->get_gc_func)
-	fg_gc = info->get_gc_func (context, &fg_color, gc);
+	fg_gc = info->get_gc_func (context, &fg_color.color, gc);
       else
 	fg_gc = gc;
 
@@ -1294,7 +1365,7 @@ pango_x_render_layout_line (Display          *display,
 
       if (bg_set && info->get_gc_func)
 	{
-	  GC bg_gc = info->get_gc_func (context, &bg_color, gc);
+	  GC bg_gc = info->get_gc_func (context, &bg_color.color, gc);
 
 	  XFillRectangle (display, drawable, bg_gc,
 			  x + (x_off + logical_rect.x) / PANGO_SCALE,
@@ -1686,7 +1757,7 @@ font_struct_get_ligatures (PangoFontMap *fontmap,
       list = g_list_next (list);
     }
 
-  g_list_free (list);
+  g_list_free (list_start);
   
   info->n_ligs = n_linfo;
   info->ligs = linfo;
